/*****************************************************
ATmega8
8,000000 MHz
*****************************************************/

#include <mega8.h>
#include <delay.h>
#include <stdlib.h>
#include <stdio.h>
#include <n3310lcd.c>

#asm                             //    
        .equ __w1_port=0x15 ;PORTA 
        .equ __w1_bit=5
#endasm                 

#include <1wire.h> 
#include <ds18b20.h>

#define MAX_DS18B20 9

/**************************************************************************/
unsigned char Mode;

#define Manual 0                            //     
#define Auto 1                              //  +
#define Max 2                               //  

#define UP_Button (!PINC.1)    //    
#define DOWN_Button (!PINC.2)  //    
#define MODE_Button (!PINC.3)  //    
/************************************************************************/

/*********************************************************************/
#define FIRST_ADC_INPUT 0 
#define LAST_ADC_INPUT 0 
unsigned char adc_data[LAST_ADC_INPUT-FIRST_ADC_INPUT+1]; 
#define ADC_VREF_TYPE 0x20 
interrupt [ADC_INT] void adc_isr(void) //   ADC  CVAVR 
{ 
static unsigned char input_index=0; 
// Read the 8 most significant bits 
// of the AD conversion result 
adc_data[input_index]=ADCH; 
// Select next ADC input 
if (++input_index > (LAST_ADC_INPUT-FIRST_ADC_INPUT)) 
input_index=0; 
ADMUX=(FIRST_ADC_INPUT | (ADC_VREF_TYPE & 0xff))+input_index; 
// Delay needed for the stabilization of the ADC input voltage 
delay_us(10); 
// Start the AD conversion 
ADCSRA|=0x40; 
}
/**********************************************************************/
int x1, x2;
unsigned char ds18b20_devices;
unsigned char ds18b20_rom_codes[MAX_DS18B20][9];

unsigned int ind;             //   . .

/* *********/
int temper[9]; 
int H;
void temperature(void) //     - 
{
for (H=0;H<ds18b20_devices;H++){
temper[H]=ds18b20_temperature(&ds18b20_rom_codes[H][0]); 
if (temper[H]>1000){               //    1000 
              temper[H]=4096-temper[H];            //   4096 
              temper[H]=-temper[H];                //   ""
                    } 
}
}
/******************************/


/* */
void init_pwm (void)
{
  TCCR1A=(1<<COM1A1)|(1<<WGM10); //  OC1A ,  OCR1A==TCNT1,  
  TCCR1B=(1<<CS10);		 //= /1
  OCR1A=0x01;			//  
}
/***************************/


void main(void)
{
char i;

unsigned char Mode_Change_Enabled = 1;    //   

// Port D initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=P State2=P State1=P State0=T 
PORTD=0x00;
DDRD=0x00;

// Port C initialization
// Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State6=T State5=T State4=T State3=P State2=P State1=P State0=T 
PORTC=0x0E;
DDRC=0x00;

DDRB=0x02;   			// PB1 (OC1A)  


// Determine the number of DS1820 devices 
// connected to the 1 Wire bus 
ds18b20_devices=w1_search(0xf0,ds18b20_rom_codes);

LcdInit(); //  
init_pwm(); // PWR


// ADC initialization
// ADC Clock frequency: 125,000 kHz
// ADC Voltage Reference: AREF pin
ADMUX=FIRST_ADC_INPUT | (ADC_VREF_TYPE & 0xff); 
ADCSRA=0xCC; 



while (1)
      {
        delay_ms(50);   //          -     
        //      50mS,         
        if (Mode_Change_Enabled)
        {
            if (MODE_Button)                        //      
            {  
                if ((++Mode) == 3) Mode = 0;   //    
                Mode_Change_Enabled = 0;       //       
            };
        }
        //   ,    
        else if (!MODE_Button) {Mode_Change_Enabled = 1;};
        
        switch (Mode)                                   //   
        {
            case Manual:
            { LcdClear();
                sprintf (lcd_buf, "");LcdString (5,1);
                                               
               
               if (UP_Button)                           //     
                    {   
                        if (OCR1A < 255) OCR1A++;
                        
                    };
                if (DOWN_Button)                      //     
                    {
                        if (OCR1A) OCR1A--;
                    };
                
                x1=((OCR1A*100)/254);
                x2=x1;
                x1=(x1*83)/100;
                LcdLine (1,34,x1,34,2);
                LcdLine (1,35,x1,35,2);
                LcdLine (1,36,x1,36,2);
                LcdLine (0,32,0,37,2);
                LcdLine (83,32,83,37,2);
                LcdLine (0,32,83,32,2);
                LcdLine (0,37,83,37,2);
                sprintf (lcd_buf, " <          >");LcdString (1,6);
                sprintf (lcd_buf, "%i",x2);LcdStringBold (6,3);    
                LcdUpdate();
                delay_ms(10);
                LcdMode (2);
                break;
            }; 
            
            
            case Auto:                              //    
            {   ind=((100.0*(adc_data[0]<<2))/204)*4;
                LcdClear();
                LcdContrast (0x00);
               
                sprintf (lcd_buf, " ( ");LcdString (1,1); 

                temperature();

                /* */
                sprintf (lcd_buf, " )  IN %i'C", temper[0]); LcdString(1,2);  //  1 

                /* */
                sprintf (lcd_buf, " *  OUT %i'C", temper[1]); LcdString(1,3); //  2 

                #asm("sei")
                sprintf (lcd_buf, "");LcdString (1,5); 
                sprintf (lcd_buf, "  %u.%uV",ind/100,ind%100);LcdString (1,6);  // 

                LcdBar(65, 27, 75, 45, 10*(3+1));   //  
                LcdUpdate();
                delay_ms(10);
                LcdMode (2);
                
                break;          
            };
            
            
            
            case Max:    
            {  
            LcdClear();
            sprintf (lcd_buf, " 3",); LcdString(1,2);  //    
            LcdUpdate();
            delay_ms(10);
            LcdMode (2);
            break; 
            };
        };
      }

}